unit Reindex;

interface

uses
  Classes, DBTables, SysUtils, Dialogs, DbiProcs, DB;

const
  NumFiles  = 1;  {number of databases declared below}
  MDXoffset = 28; {offset address of MDX status in xBase files}
  AliasName = 'DB_TEST1'; {name of the alias used}

type
  TIndexList = record
    DBFname: string[8];
    TagNames: TStringList; {contains names of all Tags}
    TagExpr:  TStringList; {contains the expression of all Tags}
  end;
  procedure LoadIndexes;  {Load the index list}
  procedure IndexFiles;      {Delete all MDX and recreate them}
  procedure FreeIndexes;  {Release the index list from memory}
  procedure DBFnoMDX(cDBFfile: string); {Reset the 'MDX' byte to 'No MDX'}
  function GetAliasPath(sAlias: string): string; {get path for ALIAS}
var
  IndexList: array[1..NumFiles] of TIndexList;

implementation

procedure LoadIndexes;
{ load all index description into the array }
begin
  {list of all databases and associated indexes}
  with IndexList[1] do begin
    DBFname  := 'TESTDATA';  TagNames := TStringList.Create; TagExpr  := TStringList.Create;
    TagNames.Add('SLOT');     TagExpr.Add('DTOS(CRT_DATE)+CRT_CODE');
    TagNames.Add('CRT_DATE'); TagExpr.Add('CRT_DATE');
    TagNames.Add('CRT_CODE'); TagExpr.Add('CRT_CODE');
  end;
  {add any other indexes/databases}
end;

procedure FreeIndexes;
{release the memory allocated to the List of Indexes}
var
  n: integer;
begin
  for n := 1 to NumFiles do
  begin
    IndexList[n].TagNames.Free;
    IndexList[n].TagExpr.Free;
  end;
end;

procedure IndexFiles;
{ recreate all indexes }
var
  n, nTag: integer;
  Table1: TTable;
  DBpath: string;
begin
  {get path}
  DBpath := GetAliasPath(AliasName);

  {Reindex}
  Table1 := TTable.Create(nil);  {create a Table object}
  try
    for n := 1 to NumFiles do
    begin
      {zap MDX if it exist}
      DeleteFile(DBpath+'\'+IndexList[n].DBFname+'.MDX');
      DBFnoMDX(DBpath+'\'+IndexList[n].DBFname+'.DBF');

      {open database}
      Table1.DatabaseName := AliasName;
      Table1.TableName := IndexList[n].DBFname+'.DBF';
      Table1.Exclusive := True;
      Table1.Open; {fail if it is opened by other process}
      try
        {for each tag in the array, add it to the MDX}
        for nTag := 0 to (IndexList[n].TagNames.Count-1) do
        begin
          if (Pos('(',IndexList[n].TagExpr.Strings[nTag]) > 0) or
             (Pos('+',IndexList[n].TagExpr.Strings[nTag]) > 0) then
            Table1.AddIndex(IndexList[n].TagNames.Strings[nTag],
                            IndexList[n].TagExpr.Strings[nTag],
                            [ixExpression])
          else
            Table1.AddIndex(IndexList[n].TagNames.Strings[nTag],
                            IndexList[n].TagExpr.Strings[nTag],
                            []);
        end;
      finally
        Table1.Close;
      end;
    end;
  finally
    Table1.Free;         {release the Table object}
  end;
end;

procedure DBFnoMDX(cDBFfile: string);
{reset the byte stating there is an MDX associated with the given DBF}
var
  handle: integer;
  buffer: char;
begin
  {open file R/W}
  handle := FileOpen(cDBFfile, 2);
  try
    {change bytes to 'No MDX'}
    FileSeek(handle, MDXoffset, 0);
    FileRead(handle, buffer, 1);
    if (buffer = #1) then
    begin
      FileSeek(handle, MDXoffset, 0);
      buffer := #0;
      FileWrite(handle, buffer, 1);
    end;
  finally
    {close file}
    FileClose(handle);
  end;
end;

function GetAliasPath(sAlias: string): string;
{Get the physical path pointed to by the ALIAS}
var
  Database: TDatabase;
  pszDir: PChar;
begin
  Database := TDatabase.Create(nil);                 {allocate memory}
  pszDir := StrAlloc(255);
  try
    Database.AliasName := sAlias;
    Database.DatabaseName := 'TEMP';   {requires a name -- is ignored}
    Database.Connected := True;    {connect without opening any table}
    DbiGetDirectory(Database.Handle, True, pszDir);     {get the dir.}
    GetAliasPath := StrPas(pszDir);              {convert to a string}
    Database.Connected := False;                          {disconnect}
  finally
    Database.Free;                                       {free memory}
    StrDispose(pszDir);
  end;
end;

end.

